;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

@CLS=0A2A
@Dump=BD00
@LastK=4025
@Seed=4032
@DFile=400C
@Scroll21=487D

ORG 5D8B			;23947

;================================
; LIFE Analysis
;================================
@LIFE
	CALL CLS
	CALL Gen0
@Analysis
	LD B,00			;B counts the no. of cell positions.
	LD DE,Dump		;DE stores the start of the working area used to compute the next gen.
	LD HL,(DFile)
	PUSH HL			;Stack the start of the display-file.
@Copy
	LD A,(HL)		;Copy the current generation (but not newlines) to the working space.
	INC HL
	CP 76
	JR Z Copy
	LD (DE),A
	INC DE
	DJNZ Copy
	LD DE,Dump		;Stack the start of the dump.
	PUSH DE
@NextCell
	LD C,00			;C counts the number of neighbours a particular cell has.
	POP DE
	POP HL			;Skip over the next character in the display file if it is a newline.
	LD A,(HL)
	CP 76
	JR NZ Valid
	INC HL
@Valid
	PUSH HL
	EX DE,HL		;Store the position within the dump
	PUSH HL			;of the cell being examined in HL,
				;and also stack it.
	LD DE,Table		;Point DE to table of displacements.
@NextDis
	LD A,(DE)		;Find displacement.
	CP 76			;If this "displacement" is 76 we
	JR Z Counted		;have reached the end of the table.
	INC DE			;Point DE to next item in table.
	ADD A,L			;Find neighbouring cell-position.
	LD L,A
	LD A,(HL)		;Is there a cell there?
	CP B4
	JR NZ NextDis
	INC C			;Increase count if so.
	JR NextDis
@Counted
	POP HL			;Retrieve cell position.
	LD A,C
	CP 02			;Are there less than two neighbours?
	JR C NoCell		;If so no cell appears.
	CP 04			;Are there four or more?
	JR NC NoCell		;If so no cell appears.
	CP 03			;Are there precisely three?
	JR Z Cell		;If so, a cell does appear.
	LD A,(HL)
	JR Put
@Cell
	LD A,B4
	JR Put
@NoCell
	LD A,80			;A now contains the right character.
@Put
	EX (SP),HL		;Retrieve print position.
	LD (HL),A		;Print character.
	INC HL			;Move print position along line.
	EX (SP),HL		;Retrieve cell-position.
	INC HL			;Look at next cell-position.
	PUSH HL			;Stack the position.
	LD A,L			;Check the value of L to find out
	AND A			;whether or not we have printed the
	JR NZ NextCell		;last cell-position.
	POP HL			;Restore the stack to its original
	POP HL			;state and return to BASIC.
	XOR A
	IN A,FE
	CP 7E			;"Q"uit?
	RET Z
	CP 77			;"R"estart?
	JP Z Life+3
	JR Analysis
	LD C,B
	RET
;================================
; Gen0
;================================
@Gen0
	CALL Scroll21
	LD C,10			;C counts the number of rows printed.
@NewRow
	LD B,10			;B counts the number of columns.
@Next
	LD HL,(Seed)		;This next section generates a random number.
	LD D,H
	LD E,L
	ADD HL,HL
	ADD HL,HL
	ADD HL,DE
	ADD HL,HL
	ADD HL,HL
	ADD HL,HL
	ADD HL,DE
	LD (Seed),HL		;The new random-number-seed is stored.
	LD A,H
@Density
	CP C4			;Decide which character to print,
	JR C Black		;based on choice of random number.
	LD A,B4
	JR Char
@Black
	LD A,80
@Char
	RST 10			;Print this character.
	DJNZ Next		;Same for the next character in the row.
	LD A,76
	RST 10
	DEC C		
	JR NZ NewRow		;Same for next row.
	RET			;Generation zero printed completely.
	00
;================================
; Table
;================================
@Table				;NW-N-NE-E-SE-S-SW-W
	EF 01 01 10		;Data representing the displacements
	10 FF FF F0		;of the neighbouring squares.