;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

@PushReg=0220			;Push reg then start Display-1
@Disp3=0292			;DISPLAY BLANK LINES routine 
@DispRout=0281			;VIDEO-1 routine
@PopReg=02A4			;Pop reg then return control to program
@Laddr=09D8			;LINE ADDRESS subroutine
@Break=0F46			;Break-1 routine
@CHAddr=4016			;Next chr addr
@Seed=4032			;16 bit parameter passing via RAND XXXX
@Frames=4034			;SYS VAR: Frames counter
@HRDFile=2000			;Hires DFile

;================================
; WRX Video driver by Wilf Rigter
; WRX16K - 1996
; Annotations by Wilf Rigter
;================================
@WRX
	LD IX,HR		;Simple start of the hres mode
	RET
@LBuf
	LD R,A			;Load HFILE address LSB
	80 80 80 80		;32 NOPs = 256 pixels
	80 80 80 80 		;00=Black on white display
	80 80 80 80 		;80=White on black display
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	RET NZ			;Always returns
@HR    
	LD B,06			;Delay
@HR0
	DJNZ HR0
	NOP			;More delay fine tuning
	NOP
	NOP
	NOP
	NOP
	DEC B
@HDAddr
	LD HL,HRDFile		;HRDFILE
	LD DE,20		;32 bytes per line
	LD B,C0			;192 lines per hires screen
@HR1
	LD A,H			;Get HFILE address MSB
	LD I,A			;Load MSB into I register
	LD A,L			;Get HFILE address LSB
	CALL C089		;CALL Lbuf+8000
	ADD HL,DE		;Next line
	DEC B			;Dec line counter
	JP NZ HR1		;Last line
@HR2   
	CALL Disp3		;Return to application program
	CALL PushReg		;Extra register PUSH and VSYNC
               			;This code segment is optional
@Brake
	CALL Break		;Check break key
	LD A,1E			;Restore pattern table pointer
	LD I,A
	JR NC Stop		;Skip the HR vector load if BREAK
	LD IX,HR		;Load the HR vector
@Stop
	JP PopReg		;Return to application program
;================================
@JumpTable
	JR WRX			;16608
	JR Lores		;16610
	JR HRCLS		;16612
	JR Invert		;16614
	JR HRPrint		;16616
;================================
; Restore lores display
;================================
@Lores
	HALT			;Wait for an interrupt - used to sync the changeover!
	LD A,(Frames) 		;Wait for the next new tv frame
	LD C,A
@Sync
	LD A,(Frames)
	CP C
	JR Z Sync
	LD A,1E		 	;Reset the "I" Register
	LD I,A		 	;to the ROM default (30 decimal)
	LD IX,DispRout	 	;Pointer to the ROM display routine set!
@Ret0
	LD BC,0000
	RET
;================================
; Clear the HRDFile
;================================
@HRCLS
	LD HL,HRDFILE		;Source=HRDFile address
	LD (HL),00		;Set to 0
	LD D,H
	LD E,L
	INC DE			;Destination=HL+1
	LD BC,17FF		;Byte count=HRDFile size-1
	LDIR			;Clear the HRDfile
	RET			;RET to BASIC, BC=0000
;================================
; WRX invert screen
;================================
@Invert
	PUSH HL			;Save main reg, EXX doesn't seem to work for some reason
	PUSH DE
	PUSH BC
	LD HL,Lbuf+2		;Source=WRX hires line buffer+2
	LD BC,001F		;Byte count, 31
	LD A,(HL) 		;Invert the value
	ADD A,80 		;by adding 128
	LD (HL),A		;Load the inverted value 
	LD D,H 
	LD E,L 
	INC DE 			;Set destination
	LDIR 
	POP BC			;Restore main reg
	POP DE
	POP HL
	RET
;================================
; Print text file
;================================
@HRPrint
	CALL HRCLS		;Clear HRDFile
;	CALL WRX		;Start WRX
@Print64
	LD HL,0003		;Locate line 3
	CALL LAddr		;Use ROM routine to find line address
	LD BC,000D		;Offset to Font64 base address
	ADD HL,BC		;HL=Font64 base address
	LD (Font+1),HL		;POKE font pointer
	LD HL,(CHAddr)		;Next chr address
	INC HL			;Skip THEN
	EX DE,HL		;DE=Text address-1
@Line64
	LD C,00			;Xcoord=0
	LD A,C			;A=Xcoord
	LD (Xcoord+1),A		;POKE Xcoord
	INC DE			;Increase text pointer
	LD A,(DE)		;Get chr to print
	CP 0C			;Newline?
	JR Z NL64		;Jump if "" chr
	CP 76			;EOF?
	JR Z Ret0		;Return if end of REM line
	CALL PrintC64		;Print chr
	INC C			;Increase Xcoord
	BIT 6,C			;C=64=EOL?
	JR Z Line64+2 		;Jump if not EOL
@NL64
	INC B			;Increase Ycoord
	LD A,18			;24 rows/screen
	CP B			;End of page?
	JR NZ Line64		;Loop until B=24
	JR Ret0			;Return when screen full
;================================
; Locate print position in HRDFile
; then print Reg A chr.
; On entry...
;  A=chr. to print
;  B=Ycoord
;  C=Xcoord
; On exit...
;  BC, DE unchanged
;================================
@PrintC64
	PUSH BC			;PUSH X/Ycoords
	PUSH DE			;PUSH text pointer
	INC B			;Ycoord must be > 0
	LD HL,HRDFile		;Get start of DFile
	DEC H
	LD DE,0100		;DE=256 bytes/row
	ADD HL,DE		
	DJNZ -3			;Add Bx256 to start of HRDFile
	LD D,E			;Add Xcoord offset to HL
	SRL C			;Xcoord=Xcoord/2
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add offset to HL
	PUSH HL 		;PUSH HRDFile addr
@Chr64
	BIT 7,A			;Uppercase chr?
	PUSH AF			;PUSH Z flag
	JR Z +2			;No, move on
	SUB 80	 		;Subtract 128 from character code
;FindChr
	LD C,A			;BC=chr code 
@Font
	LD HL,0000		;POKEd 64 colum fony base address
	RL C 			;Code*2
	RL B
	RL C 			;Code*4
	RL B 
	RL C 			;Code*8
	RL B 
	ADD HL,BC 		;HL=Pattern pointer
	EX DE,HL 		;DE=Pattern pointer
	POP AF			;POP Z flag
	POP HL 			;POP HRDFile addr
@Xcoord
	LD A,00			;POKEd Xcoord, 0 to 63
	RRCA			;C=0=Left chr   C=1=Rite chr			
	LD B,08 		;Set counter
@Row64
	PUSH BC 		;PUSH counter
	PUSH AF 		;PUSH left/rite flag
	LD A,(DE) 		;Get Font pattern
	LD B,(HL) 		;Get HRDFile pattern
	JR Z LeftLower		;Results of AND A Z=Lowercase NZ=Uppercase
	JR C ShiftRite		;
	AND F0 			;AND 11110000, A=Uppercase pattern
@LeftChr
	PUSH AF			;PUSH Font pattern 
	LD A,B 			;B=HRDFile pattern
	AND 0F 			;AND 00001111, A=Right Nybble of HRDFile pattern
@RiteChr
	LD B,A 			;B=Right Nybble of HRDFile pattern
	POP AF 			;POP Font pattern
	OR B 			;Merge font pattern with HRDFile pattern
	LD (HL),A 		;Print results
	INC DE 			;Increase font pointer
	LD BC,0020 		;32 bytes/row
	ADD HL,BC 		;Next HRDFile pixel row
	POP AF 			;POP left/rite flag
	POP BC 			;POP counter
	DJNZ Row64		;Loop until B=0
	POP DE			;POP text pointer
	POP BC			;POP X/Ycoords
	RET
@ShiftRite
	SRL A 			;Shift font pattern right
	SRL A 
	SRL A 
	SRL A 
	PUSH AF 		;PUSH shifted pattern
	LD A,B 			;A=HRDFile pattern
	AND F0 			;AND 11110000, A=Left font pattern
	JR RiteChr
@LeftLower
	JR C RiteLower
	SLA A 			;Shift font pattern left
	SLA A 
	SLA A 
	SLA A 
	JR LeftChr
@RiteLower
	AND 0F 
	JR ShiftRite+8